@@ -37,6 +37,12 @@ class KCMKScreen : public KQuickAddons::ManagedConfigModule
     Q_PROPERTY(bool xwaylandClientsScale READ xwaylandClientsScale WRITE setXwaylandClientsScale NOTIFY xwaylandClientsScaleChanged)
 
 public:
+    enum InvalidConfigReason {
+        NoEnabledOutputs,
+        ConfigHasGaps,
+    };
+    Q_ENUM(InvalidConfigReason)
+
     explicit KCMKScreen(QObject *parent, const KPluginMetaData &data, const QVariantList &args);
     ~KCMKScreen() override = default;
 
@@ -73,8 +79,7 @@ public:
     bool orientationSensorAvailable() const;
     bool tabletModeAvailable() const;
 
-    Q_INVOKABLE void forceSave();
-    void doSave(bool force);
+    void doSave();
     Q_INVOKABLE void revertSettings();
     Q_INVOKABLE void requestReboot();
 
@@ -95,7 +100,7 @@ Q_SIGNALS:
     void autoRotationSupportedChanged();
     void orientationSensorAvailableChanged();
     void tabletModeAvailableChanged();
-    void dangerousSave();
+    void invalidConfig(InvalidConfigReason reason);
     void errorOnSave();
     void globalScaleWritten();
     void outputConnect(bool connected);
@@ -112,6 +117,7 @@ private:
 
     void configReady(KScreen::ConfigOperation *op);
     void continueNeedsSaveCheck(bool needs);
+    void checkConfig();
 
     std::unique_ptr<ConfigHandler> m_configHandler;
     OrientationSensor *m_orientationSensor;
