@@ -20,8 +20,11 @@
 #include <kscreen/configmonitor.h>
 #include <kscreen/getconfigoperation.h>
 #include <kscreen/log.h>
+#include <kscreen/mode.h>
 #include <kscreen/output.h>
+#include <kscreen/screen.h>
 #include <kscreen/setconfigoperation.h>
+#include <kscreendpms/dpms.h>
 
 #include <KActionCollection>
 #include <KGlobalAccel>
@@ -31,6 +34,7 @@
 #include <QAction>
 #include <QGuiApplication>
 #include <QOrientationReading>
+#include <QScreen>
 #include <QShortcut>
 #include <QTimer>
 
@@ -82,6 +86,17 @@ KScreenDaemon::KScreenDaemon(QObject *parent, const QList<QVariant> &)
     KScreen::Log::instance();
     qMetaTypeId<KScreen::OsdAction>();
     QMetaObject::invokeMethod(this, "getInitialConfig", Qt::QueuedConnection);
+
+    auto dpms = new KScreen::Dpms(this);
+    connect(dpms, &KScreen::Dpms::modeChanged, this, [this](KScreen::Dpms::Mode mode, QScreen *screen) {
+        if (m_monitoredConfig && m_monitoredConfig->data() && screen->geometry() == m_monitoredConfig->data()->primaryOutput()->geometry()) {
+            if (mode == KScreen::Dpms::On) {
+                m_orientationSensor->setEnabled(m_monitoredConfig->autoRotationRequested());
+            } else {
+                m_orientationSensor->setEnabled(false);
+            }
+        }
+    });
 }
 
 void KScreenDaemon::getInitialConfig()
@@ -133,18 +148,20 @@ void KScreenDaemon::init()
     connect(m_lidClosedTimer, &QTimer::timeout, this, &KScreenDaemon::disableLidOutput);
 
     connect(Device::self(), &Device::lidClosedChanged, this, &KScreenDaemon::lidClosedChanged);
-    connect(Device::self(), &Device::resumingFromSuspend, this, [&]() {
+    connect(Device::self(), &Device::resumingFromSuspend, this, [this]() {
         KScreen::Log::instance()->setContext(QStringLiteral("resuming"));
+        m_orientationSensor->setEnabled(m_monitoredConfig->autoRotationRequested());
         qCDebug(KSCREEN_KDED) << "Resumed from suspend, checking for screen changes";
         // We don't care about the result, we just want to force the backend
         // to query XRandR so that it will detect possible changes that happened
         // while the computer was suspended, and will emit the change events.
         new KScreen::GetConfigOperation(KScreen::GetConfigOperation::NoEDID, this);
     });
-    connect(Device::self(), &Device::aboutToSuspend, this, [&]() {
+    connect(Device::self(), &Device::aboutToSuspend, this, [this]() {
         qCDebug(KSCREEN_KDED) << "System is going to suspend, won't be changing config (waited for "
                               << (m_lidClosedTimer->interval() - m_lidClosedTimer->remainingTime()) << "ms)";
         m_lidClosedTimer->stop();
+        m_orientationSensor->setEnabled(false);
     });
 
     connect(Generator::self(), &Generator::ready, this, [this] {
@@ -286,6 +303,11 @@ void KScreenDaemon::setAutoRotate(bool value)
     m_orientationSensor->setEnabled(value);
 }
 
+bool KScreenDaemon::isAutoRotateAvailable()
+{
+    return m_orientationSensor->available();
+}
+
 void KScreenDaemon::showOSD()
 {
     auto call = m_osdServiceInterface->showActionSelector();
