@@ -1,17 +1,20 @@
 /*
     SPDX-FileCopyrightText: 2012 Alejandro Fiestas Olivares <afiestas@kde.org>
     SPDX-FileCopyrightText: 2021 David Redondo <kde@david-redondo.de>
+    SPDX-FileCopyrightText: 2022 Nate Graham <nate@kde.org>
 
     SPDX-License-Identifier: GPL-2.0-or-later
 */
 
+#include <cmath>
+
 #include "generator.h"
 #include "device.h"
 #include "kscreen_daemon_debug.h"
 #include "output.h"
 #include <QRect>
 
-#include <kscreen/config.h>
+#include <kscreen/screen.h>
 
 #if defined(QT_NO_DEBUG)
 #define ASSERT_OUTPUTS(outputs)
@@ -27,6 +30,18 @@
     }
 #endif
 
+// The industry-standard "normal" 1x scale desktop monitor DPI value since forever
+static const int targetDpiDesktop = 96;
+
+// Higher because laptop screens are smaller and used closer to the face
+static const int targetDpiLaptop = 125;
+
+// Because phone and tablet screens are even smaller and used even closer
+static const int targetDpiMobile = 136;
+
+// Round calculated ideal scale factor to the nearest quarter
+static const int scaleRoundingness = 4;
+
 Generator *Generator::instance = nullptr;
 
 bool operator<(const QSize &s1, const QSize &s2)
@@ -71,7 +86,6 @@ KScreen::ConfigPtr Generator::idealConfig(const KScreen::ConfigPtr &currentConfi
 {
     Q_ASSERT(currentConfig);
 
-    //     KDebug::Block idealBlock("Ideal Config");
     KScreen::ConfigPtr config = currentConfig->clone();
 
     disableAllDisconnectedOutputs(config->outputs());
@@ -89,18 +103,17 @@ KScreen::ConfigPtr Generator::idealConfig(const KScreen::ConfigPtr &currentConfi
     }
 
     if (connectedOutputs.count() == 1) {
-        singleOutput(connectedOutputs);
+        singleOutput(config);
         return config;
     }
 
     if (isLaptop()) {
-        laptop(connectedOutputs);
+        laptop(config);
         return fallbackIfNeeded(config);
     }
 
     qCDebug(KSCREEN_KDED) << "Extend to Right";
-    extendToRight(connectedOutputs);
-
+    extendToRight(config, connectedOutputs);
     return fallbackIfNeeded(config);
 }
 
@@ -119,10 +132,10 @@ KScreen::ConfigPtr Generator::fallbackIfNeeded(const KScreen::ConfigPtr &config)
             KScreen::OutputList connectedOutputs = config->connectedOutputs();
             if (connectedOutputs.isEmpty()) {
                 return config;
+            } else {
+                config->setPrimaryOutput(connectedOutputs.first());
+                cloneScreens(config);
             }
-            const auto &firstConnectedOutputKey = connectedOutputs.constBegin().key();
-            connectedOutputs.value(firstConnectedOutputKey)->setPrimary(true);
-            cloneScreens(connectedOutputs);
         }
     } else {
         newConfig = config;
@@ -139,7 +152,6 @@ KScreen::ConfigPtr Generator::fallbackIfNeeded(const KScreen::ConfigPtr &config)
 
 KScreen::ConfigPtr Generator::displaySwitch(DisplaySwitchAction action)
 {
-    //     KDebug::Block switchBlock("Display Switch");
     KScreen::ConfigPtr config = m_currentConfig;
     Q_ASSERT(config);
 
@@ -151,13 +163,13 @@ KScreen::ConfigPtr Generator::displaySwitch(DisplaySwitchAction action)
 
     // There's not much else we can do with only one output
     if (connectedOutputs.count() < 2) {
-        singleOutput(connectedOutputs);
+        singleOutput(config);
         return config;
     }
 
     // We cannot try all possible combinations with two and more outputs
     if (connectedOutputs.count() > 2) {
-        extendToRight(connectedOutputs);
+        extendToRight(config, connectedOutputs);
         return config;
     }
 
@@ -175,7 +187,7 @@ KScreen::ConfigPtr Generator::displaySwitch(DisplaySwitchAction action)
         }
         if (!embedded) {
             // If all else fail take the first screen
-            embedded = connectedOutputs.constBegin().value();
+            embedded = connectedOutputs.first();
         }
     }
     // Just to be sure
@@ -185,8 +197,8 @@ KScreen::ConfigPtr Generator::displaySwitch(DisplaySwitchAction action)
 
     if (action == Generator::Clone) {
         qCDebug(KSCREEN_KDED) << "Cloning";
-        embedded->setPrimary(true);
-        cloneScreens(connectedOutputs);
+        config->setPrimaryOutput(embedded);
+        cloneScreens(config);
         return config;
     }
 
@@ -229,20 +241,16 @@ KScreen::ConfigPtr Generator::displaySwitch(DisplaySwitchAction action)
     case Generator::TurnOffEmbedded: {
         qCDebug(KSCREEN_KDED) << "Turn off embedded (laptop)";
         embedded->setEnabled(false);
-        embedded->setPrimary(false);
-
         external->setEnabled(true);
-        external->setPrimary(true);
+        config->setPrimaryOutput(external);
         return config;
     }
     case Generator::TurnOffExternal: {
         qCDebug(KSCREEN_KDED) << "Turn off external screen";
         embedded->setPos(QPoint(0, 0));
         embedded->setEnabled(true);
-        embedded->setPrimary(true);
-
         external->setEnabled(false);
-        external->setPrimary(false);
+        config->setPrimaryOutput(embedded);
         return config;
     }
     case Generator::ExtendToRight: {
@@ -270,15 +278,17 @@ uint qHash(const QSize &size)
     return size.width() * size.height();
 }
 
-void Generator::cloneScreens(KScreen::OutputList &connectedOutputs)
+void Generator::cloneScreens(const KScreen::ConfigPtr &config)
 {
+    KScreen::OutputList connectedOutputs = config->connectedOutputs();
+
     ASSERT_OUTPUTS(connectedOutputs);
     if (connectedOutputs.isEmpty()) {
         return;
     }
 
     QSet<QSize> commonSizes;
-    const QSize maxScreenSize = m_currentConfig->screen()->maxSize();
+    const QSize maxScreenSize = config->screen()->maxSize();
 
     Q_FOREACH (const KScreen::OutputPtr &output, connectedOutputs) {
         QSet<QSize> modeSizes;
@@ -339,33 +349,34 @@ void Generator::cloneScreens(KScreen::OutputList &connectedOutputs)
     }
 }
 
-void Generator::singleOutput(KScreen::OutputList &connectedOutputs)
+void Generator::singleOutput(KScreen::ConfigPtr &config)
 {
+    const KScreen::OutputList connectedOutputs = config->connectedOutputs();
+
     ASSERT_OUTPUTS(connectedOutputs);
     if (connectedOutputs.isEmpty()) {
         return;
     }
 
-    const auto &firstConnectedOutputKey = connectedOutputs.constBegin().key();
-    KScreen::OutputPtr output = connectedOutputs.take(firstConnectedOutputKey);
+    KScreen::OutputPtr output = connectedOutputs.first();
     if (output->modes().isEmpty()) {
         return;
     }
-    output->setEnabled(true);
-    output->setPrimary(true);
+
+    config->setPrimaryOutput(output);
     output->setPos(QPoint(0, 0));
 }
 
-void Generator::laptop(KScreen::OutputList &connectedOutputs)
+void Generator::laptop(KScreen::ConfigPtr &config)
 {
-    ASSERT_OUTPUTS(connectedOutputs)
-    if (connectedOutputs.isEmpty()) {
+    KScreen::OutputList usableOutputs = config->connectedOutputs();
+
+    ASSERT_OUTPUTS(usableOutputs)
+    if (usableOutputs.isEmpty()) {
         return;
     }
 
-    //     KDebug::Block laptopBlock("Laptop config");
-
-    KScreen::OutputPtr embedded = embeddedOutput(connectedOutputs);
+    KScreen::OutputPtr embedded = embeddedOutput(usableOutputs);
     /* Apparently older laptops use "VGA-*" as embedded output ID, so embeddedOutput()
      * will fail, because it looks only for modern "LVDS", "EDP", etc. If we
      * fail to detect which output is embedded, just use the one  with the lowest
@@ -373,106 +384,99 @@ void Generator::laptop(KScreen::OutputList &connectedOutputs)
      * See bug #318907 for further reference. -- dvratil
      */
     if (!embedded) {
-        QList<int> keys = connectedOutputs.keys();
+        QList<int> keys = usableOutputs.keys();
         std::sort(keys.begin(), keys.end());
-        const auto &firstConnectedOutputKey = connectedOutputs.constBegin().key();
-        embedded = connectedOutputs.value(firstConnectedOutputKey);
+        embedded = usableOutputs.value(keys.first());
     }
-    connectedOutputs.remove(embedded->id());
+    usableOutputs.remove(embedded->id());
 
-    if (connectedOutputs.isEmpty() || embedded->modes().isEmpty()) {
+    if (usableOutputs.isEmpty() || embedded->modes().isEmpty()) {
         qCWarning(KSCREEN_KDED) << "No external outputs found, going for singleOutput()";
-        connectedOutputs.insert(embedded->id(), embedded);
-        return singleOutput(connectedOutputs);
+        return singleOutput(config);
     }
 
-    if (isLidClosed() && connectedOutputs.count() == 1) {
+    if (isLidClosed() && usableOutputs.count() == 1) {
         qCDebug(KSCREEN_KDED) << "With lid closed";
         embedded->setEnabled(false);
-        embedded->setPrimary(false);
 
-        const auto &firstConnectedOutputKey = connectedOutputs.constBegin().key();
-        KScreen::OutputPtr external = connectedOutputs.value(firstConnectedOutputKey);
+        KScreen::OutputPtr external = usableOutputs.first();
         if (external->modes().isEmpty()) {
             return;
         }
-        external->setEnabled(true);
-        external->setPrimary(true);
+        config->setPrimaryOutput(external);
         external->setPos(QPoint(0, 0));
-
         return;
     }
 
-    if (isLidClosed() && connectedOutputs.count() > 1) {
+    if (isLidClosed() && usableOutputs.count() > 1) {
         qCDebug(KSCREEN_KDED) << "Lid is closed, and more than one output";
         embedded->setEnabled(false);
-        embedded->setPrimary(false);
 
-        extendToRight(connectedOutputs);
+        extendToRight(config, usableOutputs);
         return;
     }
 
     qCDebug(KSCREEN_KDED) << "Lid is open";
+
     // If lid is open, laptop screen should be primary
     embedded->setPos(QPoint(0, 0));
-    embedded->setPrimary(true);
     embedded->setEnabled(true);
-
     int globalWidth = embedded->geometry().width();
-    KScreen::OutputPtr biggest = biggestOutput(connectedOutputs);
+
+    KScreen::OutputPtr biggest = biggestOutput(usableOutputs);
     Q_ASSERT(biggest);
-    connectedOutputs.remove(biggest->id());
+    usableOutputs.remove(biggest->id());
 
     biggest->setPos(QPoint(globalWidth, 0));
     biggest->setEnabled(true);
-    biggest->setPrimary(false);
-
     globalWidth += biggest->geometry().width();
-    for (KScreen::OutputPtr output : qAsConst(connectedOutputs)) {
-        output->setEnabled(true);
-        output->setPrimary(false);
-        output->setPos(QPoint(globalWidth, 0));
 
+    for (KScreen::OutputPtr output : qAsConst(usableOutputs)) {
+        output->setEnabled(true);
+        output->setPos(QPoint(globalWidth, 0));
         globalWidth += output->geometry().width();
     }
 
     if (isDocked()) {
         qCDebug(KSCREEN_KDED) << "Docked";
-        embedded->setPrimary(false);
-        biggest->setPrimary(true);
+        config->setPrimaryOutput(biggest);
+    } else {
+        config->setPrimaryOutput(embedded);
     }
 }
 
-void Generator::extendToRight(KScreen::OutputList &connectedOutputs)
+void Generator::extendToRight(KScreen::ConfigPtr &config, KScreen::OutputList usableOutputs)
 {
-    ASSERT_OUTPUTS(connectedOutputs);
-    if (connectedOutputs.isEmpty()) {
+    ASSERT_OUTPUTS(usableOutputs);
+    if (usableOutputs.isEmpty()) {
         return;
     }
 
     qCDebug(KSCREEN_KDED) << "Extending to the right";
-    KScreen::OutputPtr biggest = biggestOutput(connectedOutputs);
-    Q_ASSERT(biggest);
 
-    connectedOutputs.remove(biggest->id());
+    KScreen::OutputPtr biggest = biggestOutput(usableOutputs);
+    Q_ASSERT(biggest);
+    usableOutputs.remove(biggest->id());
 
     biggest->setEnabled(true);
-    biggest->setPrimary(true);
     biggest->setPos(QPoint(0, 0));
-
     int globalWidth = biggest->geometry().width();
 
-    for (KScreen::OutputPtr output : qAsConst(connectedOutputs)) {
+    for (KScreen::OutputPtr output : qAsConst(usableOutputs)) {
         output->setEnabled(true);
-        output->setPrimary(false);
         output->setPos(QPoint(globalWidth, 0));
-
         globalWidth += output->geometry().width();
     }
+
+    config->setPrimaryOutput(biggest);
 }
 
 void Generator::initializeOutput(const KScreen::OutputPtr &output, KScreen::Config::Features features)
 {
+    if (output->modes().empty()) {
+        output->setEnabled(false);
+        return;
+    }
     Output::GlobalConfig config = Output::readGlobal(output);
     output->setCurrentModeId(config.modeId.value_or(bestModeForOutput(output)->id()));
     output->setRotation(config.rotation.value_or(output->rotation()));
@@ -530,19 +534,48 @@ KScreen::ModePtr Generator::bestModeForSize(const KScreen::ModeList &modes, cons
 
 qreal Generator::bestScaleForOutput(const KScreen::OutputPtr &output)
 {
-    // if we have no physical size, we can't determine the DPI properly. Fallback to scale 1
+    // Sanity check outputs that tell us they have no physical size
     if (output->sizeMm().height() <= 0) {
         return 1.0;
     }
-    const auto mode = output->currentMode();
-    const qreal dpi = mode->size().height() / (output->sizeMm().height() / 25.4);
 
-    // if reported DPI is closer to two times normal DPI, followed by a sanity check of having the sort of vertical resolution
-    // you'd find in a high res screen
-    if (dpi > 96 * 1.5 && mode->size().height() >= 1440) {
-        return 2.0;
+    /* The eye's ability to perceive detail diminishes with distance, so objects
+     * that are closer can be smaller and their details remain equally
+     * distinguishable. As a result, each device type has its own ideal physical
+     * size of items on its screen based on how close the user's eyes are
+     * expected to be from it on average, and its target DPI value needs to be
+     * changed accordingly.
+     */
+    int outputTargetDpi;
+
+    if (output->type() != KScreen::Output::Panel) {
+        outputTargetDpi = targetDpiDesktop;
+    } else {
+        if (isLaptop()) {
+            outputTargetDpi = targetDpiLaptop;
+        } else {
+            outputTargetDpi = targetDpiMobile;
+        }
     }
-    return 1.0;
+
+    const qreal outputPixelHeight = output->currentMode()->size().height();
+    const qreal outputPhysicalHeight = output->sizeMm().height() / 25.4; // convert mm to inches
+    const qreal outputDpi = outputPixelHeight / outputPhysicalHeight;
+
+    const qreal scale = round(outputDpi / outputTargetDpi * scaleRoundingness) / scaleRoundingness;
+
+    // Sanity check for outputs with such a low pixel density that the calculated
+    // scale would be less than 1; this isn't well supported, so just use 1
+    if (scale < 1) {
+        return 1.0;
+    }
+    // The KCM doesn't support manually setting the scale higher than 3x, so limit
+    // the auto-calculated value to that
+    else if (scale > 3) {
+        return 3.0;
+    }
+
+    return scale;
 }
 
 KScreen::ModePtr Generator::bestModeForOutput(const KScreen::OutputPtr &output)
@@ -579,12 +612,10 @@ KScreen::OutputPtr Generator::biggestOutput(const KScreen::OutputList &outputs)
 
 void Generator::disableAllDisconnectedOutputs(const KScreen::OutputList &outputs)
 {
-    //     KDebug::Block disableBlock("Disabling disconnected screens");
     for (const KScreen::OutputPtr &output : outputs) {
         if (!output->isConnected()) {
             qCDebug(KSCREEN_KDED) << output->name() << " Disabled";
             output->setEnabled(false);
-            output->setPrimary(false);
         }
     }
 }
@@ -592,11 +623,9 @@ void Generator::disableAllDisconnectedOutputs(const KScreen::OutputList &outputs
 KScreen::OutputPtr Generator::embeddedOutput(const KScreen::OutputList &outputs)
 {
     for (const KScreen::OutputPtr &output : outputs) {
-        if (output->type() != KScreen::Output::Panel) {
-            continue;
+        if (output->type() == KScreen::Output::Panel) {
+            return output;
         }
-
-        return output;
     }
 
     return KScreen::OutputPtr();
