@@ -11,6 +11,8 @@
 
 #include <KScreen/Types>
 
+#include "../common/osdaction.h"
+
 class KScreenApplet : public Plasma::Applet
 {
     Q_OBJECT
@@ -21,7 +23,7 @@ class KScreenApplet : public Plasma::Applet
     Q_PROPERTY(int connectedOutputCount READ connectedOutputCount NOTIFY connectedOutputCountChanged)
 
 public:
-    explicit KScreenApplet(QObject *parent, const QVariantList &data);
+    explicit KScreenApplet(QObject *parent, const KPluginMetaData &data, const QVariantList &args);
     ~KScreenApplet() override;
 
     enum Action {
@@ -39,6 +41,8 @@ public:
 
     Q_INVOKABLE void applyLayoutPreset(KScreenApplet::Action action);
 
+    Q_INVOKABLE QVariant availableActions();
+
 Q_SIGNALS:
     void connectedOutputCountChanged();
 
